module Video_Background(
	input CLK,
	input [18:0] address,
	output reg [9:0] red,
	output reg [9:0] green,
	output reg [9:0] blue);

parameter DWIDTH = 16,
			 WORDS = 15'd19200;

wire [DWIDTH-1:0] data;

bg background_data(
  address[18:4],
  CLK,
  data);
reg is_high;

always @ (address, data) begin
	is_high = data[~address[3:0]];
	red = 0;//(is_high) ? 10'h3FF : 0;
	green = (is_high) ? 10'h3FF : 0;
	blue = 0;//(is_high) ? 10'h3FF : 0;
end

endmodule

module Video_My_Paddle(
	input [9:0] raw_paddle_x,
	input [8:0] raw_paddle_y,
	input [9:0] vga_x,
	input [8:0] vga_y,
	output oIsPaddle,
	output [9:0] red,
	output [9:0] green,
	output [9:0] blue
);
parameter HALF_WIDTH = 10'd75,
          HALF_HEIGHT = 9'd60,
			 BORDER_WIDTH = 10'd4;

assign red = (oIsPaddle) ? 10'h3FF : 0;
assign green = 0;
assign blue = 0;

reg [9:0] paddle_x;
reg [8:0] paddle_y;
// Bounds check and correct for out of boundary for paddle
always @ (raw_paddle_x, raw_paddle_y) begin
  paddle_x = raw_paddle_x;
  paddle_y = raw_paddle_y;
  if (raw_paddle_x < HALF_WIDTH) begin
    paddle_x = HALF_WIDTH;
  end
  if (raw_paddle_x > (10'd639 - HALF_WIDTH)) begin
    paddle_x = 10'd639 - HALF_WIDTH;
  end
  if (raw_paddle_y < HALF_HEIGHT) begin
    paddle_y = HALF_HEIGHT;
  end
  if (raw_paddle_y > (10'd479 - HALF_HEIGHT)) begin
    paddle_y = 10'd479 - HALF_HEIGHT;
  end
end
			 
Video_Paddle p(
  .paddle_x (paddle_x),
  .paddle_y (paddle_y),
  .vga_x        (vga_x),
  .vga_y        (vga_y),
  .HALF_WIDTH   (HALF_WIDTH),
  .HALF_HEIGHT  (HALF_HEIGHT),
  .BORDER_WIDTH (BORDER_WIDTH),
  .oIsPaddle    (oIsPaddle)
);
endmodule

module Video_His_Paddle(
	input [9:0] raw_paddle_x,
	input [8:0] raw_paddle_y,
	input [9:0] vga_x,
	input [8:0] vga_y,
	output oIsPaddle,
	output [9:0] red,
	output [9:0] green,
	output [9:0] blue
);
parameter HALF_WIDTH = 10'd75,
          HALF_HEIGHT = 9'd60,
          TRANS_HALF_WIDTH = 10'd22, // THIS IS A GUESS, NEED TO GET ACCURAT NUMBERS
          TRANS_HALF_HEIGHT = 9'd17,
			 BORDER_WIDTH = 10'd1;

assign red = 0;
assign green = 0;
assign blue = (oIsPaddle) ? 10'h3FF : 0;

reg [9:0] paddle_x;
reg [8:0] paddle_y;
// Bounds check and correct for out of boundary for paddle
always @ (raw_paddle_x, raw_paddle_y) begin
  paddle_x = raw_paddle_x;
  paddle_y = raw_paddle_y;
  if (raw_paddle_x < HALF_WIDTH) begin
    paddle_x = HALF_WIDTH;
  end
  if (raw_paddle_x > (10'd639 - HALF_WIDTH)) begin
    paddle_x = 10'd639 - HALF_WIDTH;
  end
  if (raw_paddle_y < HALF_HEIGHT) begin
    paddle_y = HALF_HEIGHT;
  end
  if (raw_paddle_y > (10'd479 - HALF_HEIGHT)) begin
    paddle_y = 10'd479 - HALF_HEIGHT;
  end
end

// Do translation to correct for depth
// Top left corner is X = 226, Y = 171 (directly measured)
// Corrected by experimentation to X = 230, Y = 174
wire [9:0] trans_paddle_x;
wire [8:0] trans_paddle_y;
assign trans_paddle_x = 10'd230 + (paddle_x >> 2) + (paddle_x >> 5);
assign trans_paddle_y = 10'd174 + (paddle_y >> 2) + (paddle_y >> 5);
			 
Video_Paddle p(
  .paddle_x (trans_paddle_x),
  .paddle_y (trans_paddle_y),
  .vga_x        (vga_x),
  .vga_y        (vga_y),
  .HALF_WIDTH   (TRANS_HALF_WIDTH),
  .HALF_HEIGHT  (TRANS_HALF_HEIGHT),
  .BORDER_WIDTH (BORDER_WIDTH),
  .oIsPaddle    (oIsPaddle)
);
endmodule

module Video_Paddle(
	input [9:0] paddle_x,
	input [8:0] paddle_y,
	input [9:0] vga_x,
	input [8:0] vga_y,
	input [9:0] HALF_WIDTH,
	input [8:0] HALF_HEIGHT,
	input [9:0] BORDER_WIDTH,
	output oIsPaddle
);
wire [9:0] left, right;
wire [8:0] top, bottom;

reg is_paddle;

// Determine whether the current VGA pixel is part of the paddle
assign left = paddle_x - HALF_WIDTH;
assign right = paddle_x + HALF_WIDTH;
assign top = paddle_y - HALF_HEIGHT;
assign bottom = paddle_y + HALF_HEIGHT;

assign oIsPaddle = is_paddle;

always @ (*) begin
  is_paddle = 0;
  if ((vga_x >= left && vga_x < left + BORDER_WIDTH) && vga_y >= top && vga_y <= bottom) begin
    is_paddle = 1;
  end
  if ((vga_x > right - BORDER_WIDTH && vga_x <= right) && vga_y >= top && vga_y <= bottom) begin
    is_paddle = 1;
  end
  if ((vga_y >= top && vga_y < top + BORDER_WIDTH) && vga_x >= left && vga_x <= right) begin
    is_paddle = 1;
  end
  if ((vga_y > bottom - BORDER_WIDTH && vga_y <= bottom) && vga_x >= left && vga_x <= right) begin
    is_paddle = 1;
  end
end

endmodule
